# ==============================================================================
#               floters - custom floating panels for houdini
#  ----------------------------------------------------------------------------
# ver 0.1.0
# dowloaded from eyevex.com
# dan@dansportfolio.com
#
# ---------------------------- known issues -----------------------------------
#
# Cant have a floating bundle list already open and launch Quick Bundler
# ==============================================================================


import hou


# -------------------------------------

def _find_quick_bundler_tab_right():
	
	result = None
	all_floating_tabs = hou.ui.floatingPaneTabs()
	
	for floating_tab in all_floating_tabs:
		
		if all([
			floating_tab.name() != 'Quick_Bundle_Builder',
			floating_tab.type().name() == 'BundleList',
			]):
			result = floating_tab
		else:
			continue
		
		return result


# -------------------------------------

def quickBundler(width=1200, height=600, pane_link=98):
	
	'''Creates a split floating panel with a FauxLiner - a tree of session nodes on the right.
	allowing multi selection from different points in the hierarchy and the bundle list pane on the left.
	
	useful for rapid bundle creation via selection of nodes from all over the session.
	
	see also: fauxLiner'''
	
	if hou.ui.findPaneTab('Quick_Bundle_Builder') != None:
		return hou.ui.findPaneTab('QB_FauxLiner').pane().floatingPanel()
		
	width     = str(width)
	height    = str(height)
	pane_link = str(pane_link)
	
	cmd_make_floater = 'pane -F -w ' + width +' '+ height + ' -n Quick_Bundle_Builder \n'
	cmd_link_pane    = 'pane -l ' + pane_link + ' Quick_Bundle_Builder \n'
	cmd_set_tab_type = 'pane -m bundlelist Quick_Bundle_Builder \n'
	cmd_format_l_tab = 'pane -a 1 -A0 -h /obj Quick_Bundle_Builder \n'
	cmd_split_pane   = 'pane -s 1 Quick_Bundle_Builder \n'
	
	hscript_cmds = [cmd_make_floater, cmd_link_pane, cmd_set_tab_type, cmd_format_l_tab, cmd_split_pane]
	hscript_string = ' '.join(hscript_cmds)
	hou.hscript(hscript_string)
	
	right_tab        = _find_quick_bundler_tab_right()
	right_tab_name   = right_tab.name()
		
	cmd_set_rt_tab_type  = 'pane -m neteditor ' + right_tab_name + ' \n'
	cmd_rename_rt_tab    = 'pane -n "QB_FauxLiner" ' + right_tab_name + ' \n'
	cmd_format_rt_tab    = 'pane -a 1 -A0 -h / QB_FauxLiner \n'
	cmd_setup_neteditor  = 'neteditor -K 0.3 -C 0.55 -p 0 -n 1 -x 0 -w 0 -k 0 -I 0 -B 1 -D 0 -L 0 -S tree QB_FauxLiner \n'
	
	hscript_cmds = [cmd_set_rt_tab_type, cmd_rename_rt_tab, cmd_format_rt_tab, cmd_setup_neteditor]
	hscript_string = ' '.join(hscript_cmds)
	hou.hscript(hscript_string)
	
	hou.ui.findPaneTab('QB_FauxLiner').setListMode(1)
	floater = hou.ui.findPaneTab('QB_FauxLiner').pane().floatingPanel()
	
	return floater
	
# -------------------------------------


def fauxLiner(width=450, height=600, pane_link=99):
	
	'''Creates a floating pane with a tree of session nodes.
	Allows multi selection from different points in the hierarchy.
	
	useful for creating bundles and selecting nodes for script processing.
	
	see also: quickBundler'''
	
	# check if it already exist
	if hou.ui.findPaneTab('FauxLiner') != None:
		return  hou.ui.findPaneTab('FauxLiner')
		
	# convert the passed numebrs to strings
	width     = str(width)
	height    = str(height)
	pane_link = str(pane_link)
	
	# build command strings
	cmd_make_floater     = 'pane -F -w ' + width +' '+ height + ' -n  FauxLiner \n'
	cmd_link_pane        = 'pane -l ' + pane_link + ' FauxLiner \n'
	cmd_set_tab_type     = 'pane -m neteditor FauxLiner \n'
	cmd_format_tab       = 'pane -a 1 -A0 -h / FauxLiner \n'
	cmd_setup_neteditor  = 'neteditor -K 0.3 -C 0.55 -p 0 -n 1 -x 0 -w 0 -k 0 -I 0 -B 1 -D 0 -L 0 -S tree  FauxLiner\n'

	# create the hscript arg string and ececute it
	hscript_cmds = [cmd_make_floater,cmd_link_pane, cmd_set_tab_type, cmd_format_tab, cmd_setup_neteditor ]
	hscript_string = ' '.join(hscript_cmds)
	hou.hscript(hscript_string)
	
	# get a py ref to the window
	floater = hou.ui.findPaneTab('FauxLiner')
	# double ensure we are in list mode
	floater.setListMode(1)
	
	return floater
	
# -------------------------------------
